package lab10;

import java.util.Iterator;

/**
 * An array implementation of a set.
 * 
 * @author CPSC170 Spring 2009
 */
public class ArraySet<T extends Comparable<T>> implements Set<T>, Iterable<T> {
	private T[] array;
	private int end;
	
	/**
	 * Initialize an empty array set.
	 */
	@SuppressWarnings("unchecked")
	public ArraySet() {
		array = (T[])(new Comparable[10]);
		end = -1;
	}

	/**
	 * @see lab10.Set#add(java.lang.Comparable)
	 */
	public void add(T element) {
		if(!contains(element)) {
			if(end == array.length-1)
				expandCapacity();
			end++;
			array[end] = element;
		}
	}

	/**
	 * Find the index of the specified element, -1 if not found.
	 * 
	 * @param target element to find
	 * @return the index of the element
	 */
	private int getIndex(T target) {
		int elementIndex = -1;
		for(int i=0; i<=end; i++) {
			if(array[i].compareTo(target) == 0)
				elementIndex = i;
		}
		return elementIndex;
	}
	
	/**
	 * @see lab10.Set#get(java.lang.Comparable)
	 */
	public T get(T element) {
		int elementIndex = getIndex(element);
		if(elementIndex == -1)
			return null;
		return array[elementIndex];
	}
	
	/**
	 * Returns true if this set contains the parameter.
	 * 
	 * @param target element to find in set
	 * @return true if element is in set, false otherwise
	 */
	private boolean contains(T target) {
		if(getIndex(target) == -1)
			return false;
		return true;
	}

	/**
	 * @see lab10.Set#iterator()
	 */
	public Iterator<T> iterator() {
		return new Iterator<T> () {
			private int current;
			
			{
				current = -1;
			}

			public boolean hasNext() {
				if(current < end)
					return true;
				return false;
			}

			public T next() {
				current++;
				return array[current];
			}

			public void remove() {
				throw new UnsupportedOperationException("Remove is not supported by the ArraySet Iterator");
			}
		};
	}

	/**
	 * @see lab10.Set#remove(java.lang.Comparable)
	 */
	public T remove(T target) {
		int elementIndex = getIndex(target);
		if(elementIndex == -1)
			return null;
		T element = array[elementIndex];
		array[elementIndex] = array[end];
		array[end] = null;
		end--;
		return element;
	}

	/**
	 * Double the size of the array used to store element data.
	 */
	@SuppressWarnings("unchecked")
	private void expandCapacity() {
		T[] tempArray = (T[])(new Comparable[array.length*2]);
		System.arraycopy(array, 0, tempArray, 0, array.length);
		array = tempArray;
	}
	
	/**
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		String string = "[";
		for(int i=0; i<end; i++)
			string += array[i] + ", ";
		if(end >= 0)
			string += array[end];
		return string + "]";
	}

}
