package lab10;

import java.util.Iterator;

/**
 * A linked implementation of a set.
 * 
 * @author CPSC170 Spring 2009
 */
public class LinkedSet<T extends Comparable<T>> implements Set<T>, Iterable<T> {
	LinkedSetNode<T> start; //pointer to first element in linked set
	
	/**
	 * Initialize an empty linked set.
	 */
	public LinkedSet() {
		start = null;
	}
	
	/**
	 * @see lab10.Set#add(java.lang.Comparable)
	 */
	public void add(T element) {
		//create new node to wrap data and update pointers to add it
		LinkedSetNode<T> newNode = new LinkedSetNode<T>(element);
		newNode.setNext(start);
		start = newNode;
	}

	/**
	 * @see lab10.Set#get(java.lang.Comparable)
	 */
	public T get(T target) {
		//iterate through elements until target is found
		for(T element : this) {
			if(element.compareTo(target) == 0)
				return element;
		}		
		return null;
	}

	/**
	 * @see lab10.Set#iterator()
	 */
	public Iterator<T> iterator() {
		// return new iterator
		return new Iterator<T> () {
			private LinkedSetNode<T> current; //pointer used by iterator to remember location

			{
				current = start;
			}

			public boolean hasNext() {
				// if not at end of list return true, false otherwise
				if(current != null)
					return true;
				return false;
			}

			public T next() {
				// return the data of the current node and set pointer to next
				T next = current.getData();
				current = current.getNext();
				return next;
			}

			public void remove() throws UnsupportedOperationException {
				// remove is not supported, throw exception
				throw new UnsupportedOperationException("Remove is not supported by the LinkedSet Iterator");
			}
		};
	}

	/**
	 * @see lab10.Set#remove(java.lang.Comparable)
	 */
	public T remove(T target) {
		// find the node in the set
		LinkedSetNode<T> current = start;
		LinkedSetNode<T> previous = null;
		while(current != null && current.getData().compareTo(target) != 0) {
			previous = current;
			current = current.getNext();
		}
		
		// if empty set, or node not in set return null
		if(current == null)
			return null;
		
		// update pointers to remove element
		if(previous == null)
			start = current.getNext();
		else
			previous.setNext(current.getNext());
		
		//return element that was removed
		return current.getData();
	}
	
	/**
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		String string = "]"; // string to store return value
		LinkedSetNode<T> node = start; // node to iterate through set
		
		// for each node add it to the output string
		while(node != null && node.getNext() != null) {
			string = ", " + node.getData() + string;
			node = node.getNext();
		}
		
		// add last node (if exists), so that it does not have trailing comma
		if(node != null)
			string = node.getData() + string;

		// add start brace and return
		string = "[" + string;
		return string;
	}
	
}
